>From 53a562565a4c441c4c057a216a67e33507e86935 Mon Sep 17 00:00:00 2001
From: Jean-Baptiste Kempf <jb@videolan.org>
Date: Wed, 16 May 2012 02:58:07 +0200
Subject: [PATCH] DirectX: fix lag...

---
 modules/audio_output/directx.c |   38 ++++++++++++++++++++++++++++++++++++--
 1 file changed, 36 insertions(+), 2 deletions(-)

diff --git a/modules/audio_output/directx.c b/modules/audio_output/directx.c
index 4cd8c99..4fae8f0 100644
--- a/modules/audio_output/directx.c
+++ b/modules/audio_output/directx.c
@@ -35,10 +35,14 @@
 #include <vlc_charset.h>
 #include <vlc_atomic.h>
 
+#include <vlc_aout_intf.h>
+#include <math.h>
+
 #include "windows_audio_common.h"
 
 #define FRAME_SIZE ((int)p_aout->format.i_rate/20) /* Size in samples */
 
+static int VolumeSet( audio_output_t *, float, bool );
 /*****************************************************************************
  * notification_thread_t: DirectX event thread
  *****************************************************************************/
@@ -85,6 +89,8 @@ struct aout_sys_t
     uint32_t i_channel_mask;
     uint32_t i_bits_per_sample;
     uint32_t i_channels;
+    volatile int m_volume;
+    int m_IsHardVolume;
 };
 
 /*****************************************************************************
@@ -282,7 +288,12 @@ static int OpenAudio( vlc_object_t *p_this )
         /* Calculate the frame size in bytes */
         aout_FormatPrepare( &p_aout->format );
         aout_PacketInit( p_aout, &p_aout->sys->packet, FRAME_SIZE );
-        aout_VolumeSoftInit( p_aout );
+        aout_VolumeHardInit (p_aout, VolumeSet, false);
+        p_aout->sys->m_IsHardVolume=1;
+
+        audio_volume_t volume = var_InheritInteger (p_aout, "volume");
+        bool mute = var_InheritBool (p_aout, "mute");
+        VolumeSet( p_aout,  (float)volume / (float)AOUT_VOLUME_DEFAULT,  mute );
     }
 
     /* Now we need to setup our DirectSound play notification structure */
@@ -792,7 +803,9 @@ static int CreateDSBuffer( audio_output_t *p_aout, int i_format,
     memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
     dsbdesc.dwSize = sizeof(DSBUFFERDESC);
     dsbdesc.dwFlags = DSBCAPS_GETCURRENTPOSITION2/* Better position accuracy */
-                    | DSBCAPS_GLOBALFOCUS;      /* Allows background playing */
+                    | DSBCAPS_GLOBALFOCUS      /* Allows background playing */
+                    | DSBCAPS_CTRLVOLUME;      /* Allows volume control */
+
 
     /* Only use the new WAVE_FORMAT_EXTENSIBLE format for multichannel audio */
     if( i_nb_channels <= 2 )
@@ -971,6 +984,12 @@ static int FillBuffer( audio_output_t *p_aout, int i_frame,
         vlc_memcpy( p_write_position, p_buffer->p_buffer, l_bytes1 );
         aout_BufferFree( p_buffer );
     }
+    /* Set volume */
+    if(p_aout->sys->m_IsHardVolume)
+    {
+        //HRESULT tt = IDirectSoundBuffer_SetVolume(p_sys->p_dsbuffer,m_volume);
+        IDirectSoundBuffer_SetVolume(p_sys->p_dsbuffer,p_aout->sys->m_volume);
+    }
 
     /* Now the data has been copied, unlock the buffer */
     IDirectSoundBuffer_Unlock( p_sys->p_dsbuffer, p_write_position, l_bytes1,
@@ -1168,3 +1187,18 @@ error:
     return VLC_SUCCESS;
 }
 
+static int VolumeSet( audio_output_t * p_aout, float volume, bool mute )
+{
+    if (volume>2.0f)
+        return 0;
+    if( mute )
+        volume = 0.;
+
+    // recode from VLC volume format (0 - 2.0) to dB
+    int vol = floor(volume*(10000.0f/2) + 0.5) + 1;
+    p_aout->sys->m_volume = floor(1085.73f * logf(vol) +0.5 ) - 10000;
+
+    return 0;
+}
+
+
-- 
1.7.10.2

